@@ -3,16 +3,16 @@
 # Copyright 2004-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Library General Public
+# modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
-# version 2 of the License, or (at your option) any later version.
+# version 2.1 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Library General Public License for more details.
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU Library General Public
+# You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
@@ -32,7 +32,6 @@ import orca.orca as orca
 import orca.debug as debug
 import orca.default as default
 import orca.orca_state as orca_state
-import orca.settings as settings
 import orca.speech as speech
 import orca.scripts.toolkits.Gecko as Gecko
 
@@ -43,6 +42,8 @@ from speech_generator import SpeechGenerator
 from script_utilities import Utilities
 import script_settings
 
+_settingsManager = getattr(orca, '_settingsManager')
+
 ########################################################################
 #                                                                      #
 # The Thunderbird script class.                                        #
@@ -464,6 +465,14 @@ class Script(Gecko.Script):
         obj = event.source
         parent = obj.parent
 
+        # Try to stop unwanted chatter when a new message is being
+        # replied to. See bgo#618484.
+        #
+        if event.source.getRole() == pyatspi.ROLE_DOCUMENT_FRAME \
+           and event.source.getState().contains(pyatspi.STATE_EDITABLE) \
+           and event.type.endswith("system"):
+            return
+
         # Speak the autocompleted text, but only if it is different
         # address so that we're not too "chatty." See bug #533042.
         #
@@ -608,7 +617,7 @@ class Script(Gecko.Script):
         self.setCaretPosition(obj, offset)
         if not script_settings.sayAllOnLoad:
             self.presentLine(obj, offset)
-        elif settings.enableSpeech:
+        elif _settingsManager.getSetting('enableSpeech'):
             self.sayAll(None)
         self._messageLoaded = False
 
