@@ -3,16 +3,16 @@
 # Copyright 2005-2009 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Library General Public
+# modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
-# version 2 of the License, or (at your option) any later version.
+# version 2.1 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Library General Public License for more details.
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU Library General Public
+# You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
@@ -58,6 +58,8 @@ from structural_navigation import StructuralNavigation
 from script_utilities import Utilities
 import script_settings
 
+_settingsManager = getattr(orca, '_settingsManager')
+
 class Script(default.Script):
 
     def __init__(self, app):
@@ -119,48 +121,42 @@ class Script(default.Script):
 
     def activate(self):
         """Called when this script is activated."""
-        self.savedreadTableCellRow = settings.readTableCellRow
-        settings.readTableCellRow = False
+        self.savedreadTableCellRow = \
+            _settingsManager.getSetting('readTableCellRow')
+        _settingsManager.setSetting('readTableCellRow', False)
 
         self.savedEnabledBrailledTextAttributes = \
-            settings.enabledBrailledTextAttributes
+            _settingsManager.getSetting('enabledBrailledTextAttributes')
 
         self.savedEnabledSpokenTextAttributes = \
-            settings.enabledSpokenTextAttributes
+            _settingsManager.getSetting('enabledSpokenTextAttributes')
 
         # Account for the differences in how OOo expresses indent, 
         # strikethrough, and margins.
         #
-        settings.allTextAttributes = \
-            settings.allTextAttributes.replace(
-            "margin:;",
-            "margin:0mm;")
+        attributes = _settingsManager.getSetting('allTextAttributes')
+        attributes.replace("margin:;", "margin:0mm;")
+        _settingsManager.setSetting('allTextAttributes', attributes)
 
-        settings.enabledBrailledTextAttributes = \
-            settings.enabledBrailledTextAttributes.replace(
-            "strikethrough:false;",
-            "strikethrough:none;")
-        settings.enabledSpokenTextAttributes = \
-            settings.enabledSpokenTextAttributes.replace(
-            "strikethrough:false;",
-            "strikethrough:none;")
+        attributes = \
+            _settingsManager.getSetting('enabledBrailledTextAttributes')
+        attributes.replace("strikethrough:false;", "strikethrough:none;")
+        attributes.replace("indent:0;", "indent:0mm;")
+        _settingsManager.setSetting('enabledBrailledTextAttributes', attributes)
 
-        settings.enabledBrailledTextAttributes = \
-            settings.enabledBrailledTextAttributes.replace(
-            "indent:0;",
-            "indent:0mm;")
-        settings.enabledSpokenTextAttributes = \
-            settings.enabledSpokenTextAttributes.replace(
-            "indent:0;",
-            "indent:0mm;")
+        attributes = _settingsManager.getSetting('enabledSpokenTextAttributes')
+        attributes.replace("strikethrough:false;", "strikethrough:none;")
+        attributes.replace("indent:0;", "indent:0mm;")
+        _settingsManager.setSetting('enabledSpokenTextAttributes', attributes)
 
     def deactivate(self):
         """Called when this script is deactivated."""
-        settings.readTableCellRow = self.savedreadTableCellRow
-        settings.enabledBrailledTextAttributes = \
-            self.savedEnabledBrailledTextAttributes
-        settings.enabledSpokenTextAttributes = \
-            self.savedEnabledSpokenTextAttributes
+        _settingsManager.setSetting('readTableCellRow',
+                                    self.savedreadTableCellRow)
+        _settingsManager.setSetting('enabledBrailledTextAttributes',
+                                    self.savedEnabledBrailledTextAttributes)
+        _settingsManager.setSetting('enabledSpokenTextAttributes',
+                                    self.savedEnabledSpokenTextAttributes)
 
     def getListeners(self):
         """Sets up the AT-SPI event listeners for this script.
@@ -175,6 +171,8 @@ class Script(default.Script):
             self.onStateChanged
         listeners["object:state-changed:checked"]           = \
             self.onStateChanged
+        listeners["object:children-changed"]                = \
+            self.onChildrenChanged
 
         return listeners
 
@@ -344,8 +342,9 @@ class Script(default.Script):
         gtk.Widget.show(self.speakCellCoordinatesCheckButton)
         gtk.Box.pack_start(tableVBox, self.speakCellCoordinatesCheckButton,
                            False, False, 0)
-        gtk.ToggleButton.set_active(self.speakCellCoordinatesCheckButton,
-                                    settings.speakCellCoordinates)
+        gtk.ToggleButton.set_active(
+            self.speakCellCoordinatesCheckButton,
+            _settingsManager.getSetting('speakCellCoordinates'))
 
         # Translators: this is an option to tell Orca whether or not it
         # should speak the span size of a table cell (e.g., how many
@@ -356,8 +355,9 @@ class Script(default.Script):
         gtk.Widget.show(self.speakCellSpanCheckButton)
         gtk.Box.pack_start(tableVBox, self.speakCellSpanCheckButton,
                            False, False, 0)
-        gtk.ToggleButton.set_active(self.speakCellSpanCheckButton,
-                                    settings.speakCellSpan)
+        gtk.ToggleButton.set_active(
+            self.speakCellSpanCheckButton,
+            _settingsManager.getSetting('speakCellSpan'))
 
         # Translators: this is an option for whether or not to speak
         # the header of a table cell in document content.
@@ -367,8 +367,9 @@ class Script(default.Script):
         gtk.Widget.show(self.speakCellHeadersCheckButton)
         gtk.Box.pack_start(tableVBox, self.speakCellHeadersCheckButton,
                            False, False, 0)
-        gtk.ToggleButton.set_active(self.speakCellHeadersCheckButton,
-                                    settings.speakCellHeaders)
+        gtk.ToggleButton.set_active(
+            self.speakCellHeadersCheckButton,
+            _settingsManager.getSetting('speakCellHeaders'))
 
         # Translators: this is an option to allow users to skip over
         # empty/blank cells when navigating tables in document content.
@@ -378,8 +379,9 @@ class Script(default.Script):
         gtk.Widget.show(self.skipBlankCellsCheckButton)
         gtk.Box.pack_start(tableVBox, self.skipBlankCellsCheckButton,
                            False, False, 0)
-        gtk.ToggleButton.set_active(self.skipBlankCellsCheckButton,
-                                    settings.skipBlankCells)
+        gtk.ToggleButton.set_active(
+                self.skipBlankCellsCheckButton,
+                _settingsManager.getSetting('skipBlankCells'))
 
         # Translators: this is the title of a panel containing options
         # for specifying how to navigate tables in document content.
@@ -403,27 +405,25 @@ class Script(default.Script):
         prefix = "orca.scripts.apps.soffice.script_settings"
 
         script_settings.speakSpreadsheetCoordinates = \
-                 self.speakSpreadsheetCoordinatesCheckButton.get_active()
+            self.speakSpreadsheetCoordinatesCheckButton.get_active()
         prefs.writelines("%s.speakSpreadsheetCoordinates = %s\n" % \
-                         (prefix, script_settings.speakSpreadsheetCoordinates))
+                        (prefix, script_settings.speakSpreadsheetCoordinates))
 
-        settings.speakCellCoordinates = \
-                 self.speakCellCoordinatesCheckButton.get_active()
-        prefs.writelines("orca.settings.speakCellCoordinates = %s\n" % \
-                         settings.speakCellCoordinates)
+        value = self.speakCellCoordinatesCheckButton.get_active()
+        _settingsManager.setSetting('speakCellCoordinates', value)            
+        prefs.writelines("orca.settings.speakCellCoordinates = %s\n" % value)
 
-        settings.speakCellSpan = self.speakCellSpanCheckButton.get_active()
-        prefs.writelines("orca.settings.speakCellSpan = %s\n" % \
-                         settings.speakCellSpan)
+        value = self.speakCellSpanCheckButton.get_active()
+        _settingsManager.setSetting('speakCellSpan', value)
+        prefs.writelines("orca.settings.speakCellSpan = %s\n" % value)
 
-        settings.speakCellHeaders = \
-                self.speakCellHeadersCheckButton.get_active()
-        prefs.writelines("orca.settings.speakCellHeaders = %s\n" % \
-                         settings.speakCellHeaders)
+        value = self.speakCellHeadersCheckButton.get_active()
+        _settingsManager.setSetting('speakCellHeaders', value)
+        prefs.writelines("orca.settings.speakCellHeaders = %s\n" % value)
 
-        settings.skipBlankCells = self.skipBlankCellsCheckButton.get_active()
-        prefs.writelines("orca.settings.skipBlankCells = %s\n" % \
-                         settings.skipBlankCells)
+        value = self.skipBlankCellsCheckButton.get_active()
+        _settingsManager.setSetting('skipBlankCells', value)
+        prefs.writelines("orca.settings.skipBlankCells = %s\n" % value)
 
     def getAppState(self):
         """Returns an object that can be passed to setAppState.  This
@@ -760,7 +760,7 @@ class Script(default.Script):
         self.updateBraille(cell)
         speech.speak(self.speechGenerator.generateSpeech(cell))
 
-        if not settings.readTableCellRow:
+        if not _settingsManager.getSetting('readTableCellRow'):
             self.speakCellName(cell.name)
 
         try:
@@ -808,7 +808,7 @@ class Script(default.Script):
                     debug.println(self.debugLevel,
                         "StarOffice.speakInputLine: contents: %s" % inputLine)
                     self.displayBrailleMessage(inputLine, \
-                      flashTime=settings.brailleFlashTime)
+                      flashTime=_settingsManager.getSetting('brailleFlashTime'))
                     speech.speak(inputLine)
             except NotImplementedError:
                 pass
@@ -1129,7 +1129,7 @@ class Script(default.Script):
         - endOffset: the end offset for this word
         """
 
-        voices = settings.voices
+        voices = _settingsManager.getSetting('voices')
 
         for i in range(startOffset, endOffset):
             if self.utilities.linkIndex(obj, i) >= 0:
@@ -1834,6 +1834,66 @@ class Script(default.Script):
 
         default.Script.onActiveDescendantChanged(self, event)
 
+    def onChildrenChanged(self, event):
+        """Called when a child node has changed.
+
+        Arguments:
+        - event: the Event
+        """
+
+        if not event.any_data or not orca_state.locusOfFocus:
+            return
+
+        if not event.type.startswith('object:children-changed:add'):
+            return
+
+        role = event.any_data.getRole()
+        if role == pyatspi.ROLE_TABLE:
+            if self.isSpreadSheetCell(event.any_data, True):
+                orca.setLocusOfFocus(event, event.any_data)
+            return
+
+        if role == pyatspi.ROLE_TABLE_CELL:
+            activeRow = self.pointOfReference.get('lastRow', -1)
+            activeCol = self.pointOfReference.get('lastColumn', -1)
+            if activeRow < 0 or activeCol < 0:
+                return
+
+            try:
+                itable = event.source.queryTable()
+            except NotImplementedError:
+                return
+
+            index = self.utilities.cellIndex(event.any_data)
+            eventRow = itable.getRowAtIndex(index)
+            eventCol = itable.getColumnAtIndex(index)
+
+            if eventRow == itable.nRows - 1 and eventCol == itable.nColumns - 1:
+                fullMessage = briefMessage = ""
+                voice = self.voices.get(settings.SYSTEM_VOICE)
+                if activeRow == itable.nRows:
+                    # Translators: This message is to inform the user that
+                    # the last row of a table in a document was just deleted.
+                    #
+                    fullMessage = _("Last row deleted.")
+                    # Translators: This message is to inform the user that
+                    # a row in a table was just deleted.
+                    #
+                    briefMessage = _("Row deleted.")
+                else:
+                    # Translators: This message is to inform the user that a
+                    # new table row was inserted at the end of the existing
+                    # table. This typically happens when the user presses Tab
+                    # from within the last cell of the table.
+                    #
+                    fullMessage = _("Row inserted at the end of the table.")
+                    # Translators: This message is to inform the user that
+                    # a row in a table was just inserted.
+                    #
+                    briefMessage = _("Row inserted.")
+                if fullMessage:
+                    self.presentMessage(fullMessage, briefMessage, voice)
+
     def onStateChanged(self, event):
         """Called whenever an object's state changes.
 
@@ -2073,7 +2133,7 @@ class Script(default.Script):
                           pyatspi.ROLE_ROOT_PANE,
                           pyatspi.ROLE_FRAME,
                           pyatspi.ROLE_APPLICATION]
-            if settings.enableEchoByWord and \
+            if _settingsManager.getSetting('enableEchoByWord') and \
                (self.utilities.hasMatchingHierarchy(event.source, rolesList) or
                 self.utilities.hasMatchingHierarchy(event.source, rolesList1)):
                 keyString, mods = self.utilities.lastKeyAndModifiers()
@@ -2172,7 +2232,7 @@ class Script(default.Script):
         # If this is a blank line, announce it if the user requested
         # that blank lines be spoken.
         if line[1] == 0 and line[2] == 0:
-            return settings.speakBlankLines
+            return _settingsManager.getSetting('speakBlankLines')
 
     def onTextInserted(self, event):
         """Called whenever text is inserted into an object.  Overridden here
